﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BioAPI
{

    /// <summary>
    /// Represents exceptional condition that occurs during the operation of a biometric system. Along with the
    /// message the object carries additional information about the error, such as the ID of the component that
    /// reported the error and the numeric code taht identifies the reason of error.
    /// </summary>
    /// <author>
    /// See CREDITS.txt
    /// </author>
    public class BioAPIException : Exception
    {

        /// <summary>
        /// Defines source of error.
        /// -Framework: The error was reported by the framework component.
        /// -BSP: The error was reported by the biometric service provider.
        /// -Unit: The error was reported by the biometric unit. 
        /// </summary>
        public enum Facility
        {
            Framework,
            BSP,
            Unit
        }

        protected Facility mSource;
        protected int mErrorCode;

        /// <summary>
        /// Creates a new instance of BioAPIException with the specified error source and the reason code.
        /// Implemented but untested.
        /// </summary>
        /// <param name="source">
        /// identifies component of the biometric system that reported an error.
        /// </param>
        /// <param name="code">
        /// numeric code that specifies the reason of error.
        /// </param>
        public BioAPIException(Facility source, int code)
        {
            Setup(source, code);
        }

        /// <summary>
        /// Creates a new instance of BioAPIException with the specified error source, reason code, and the message.
        /// Implemented but untested.
        /// </summary>
        /// <param name="source">
        /// identifies component of the biometric system taht reported an error.
        /// </param>
        /// <param name="code">
        /// numeric code that specifies the reason of error
        /// </param>
        /// <param name="message">
        /// specifies the error message
        /// </param>
        public BioAPIException(Facility source, int code, string message)
            : base(message)
        {
            Setup(source, code);
        }

        /// <summary>
        /// Creates a new instance of BioAPIException with the specified error source, reason code, message, 
        /// and cause. This constructor is used to represent an error that was caused by another exception.
        /// Implemented but untested.
        /// </summary>
        /// <param name="source">
        /// identifies component of the biometric system taht reported an error.
        /// </param>
        /// <param name="code">
        /// numeric code that specifies the reason of error
        /// </param>
        /// <param name="message">
        /// specifies the error message
        /// </param>
        /// <param name="cause">
        /// the cause (A null value is permitted, and indicates that the cause is nonexistent or unknown.)
        /// </param>
        public BioAPIException(Facility source, int code, string message, Exception cause)
            : base(message, cause)
        {
            Setup(source, code);
        }

        /// <summary>
        /// Creates a new instance of BioAPIException with the specified error source, reason code, cause, 
        /// and error message associated with the cause (if any). This constructor is used to represent an 
        /// error that was caused by another exception.
        /// </summary>
        /// <param name="source">
        /// identifies component of the biometric system taht reported an error.
        /// </param>
        /// <param name="code">
        /// numeric code that specifies the reason of error
        /// </param>
        /// <param name="cause">
        /// the cause (A null value is permitted, and indicates that the cause is nonexistent or unknown.)
        /// </param>
        public BioAPIException(Facility source, int code, System.Exception cause)
            : base("", cause)
        {
            Setup(source, code);
        }

        /// <summary>
        /// Internal method that is used to initialize the instance.
        /// Implemented but untested.
        /// </summary>
        /// <param name="source">
        /// identifies component of the biometric system taht reported an error.
        /// </param>
        /// <param name="code">
        /// numeric code that specifies the reason of error
        /// </param>
        protected void Setup(Facility source, int code)
        {
            mSource = source;
            mErrorCode = code;
        }

        /// <summary>
        /// The ID of the component of the biometric system that reported the error.
        /// Implemented but untested.
        /// </summary>
        new public Facility Source
        {
            get
            {
                return mSource;
            }
        }

        /// <summary>
        /// The numeric code that identifies the reason of the error.
        /// Implemented but untested.
        /// </summary>
        public int ErrorCode
        {
            get
            {
                return mErrorCode;
            }
        }

        public const int BioAPIFrameworkError = 0x00000000;
        public const int BioAPIBspError = 0x01000000;
        public const int BioAPIUnitError = 0x02000000;

        

        /// <summary>
        /// General system error; indicates that an operating system or internal state
        /// error has occured and the system may not be in a known state.
        /// </summary>
        public const int BioAPIErrInternalError = 0x000101;


        /// <summary>
        /// A memory error occured.
        /// </summary>
        public const int BioAPIErrMemoryError = 0x000102;


        /// <summary>
        /// An input/output function parameter or input/output field inside of a
        /// data structure is an invalid pointer.
        /// </summary>
        public const int BioAPIErrInvalidPointer = 0x000103;


        /// <summary>
        /// An input function parameter or input field in a datastructure is an invalid pointer.
        /// </summary>
        public const int BioAPIErrInvalidInputPointer = 0x000104;


        /// <summary>
        /// An output function parameter or output field in a data structure is an invalid pointer.
        /// </summary>
        public const int BioAPIErrInvalidOutputPointer = 0x000105;


        /// <summary>
        /// The function is not implemented by the biometric service provider.
        /// </summary>
        public const int BioAPIErrFunctionNotSupported = 0x000106;


        /// <summary>
        /// The operating system denied access to a required resource.
        /// </summary>
        public const int BioAPIErrOAccessDenied = 0x000107;


        /// <summary>
        /// The function failed for an unknown reason.
        /// </summary>
        public const int BioAPIErrFunctionFailed = 0x000108;


        /// <summary>
        /// An input uuid is invalid. [may occur of a component requested by the uuid is not present on the system or
        /// cannot be found.
        /// </summary>
        public const int BioAPIErrInvalidUuid = 0x000109;


        /// <summary>
        /// Version incompatibility. [may occur if the called component cannot support the version of the BioAPI
        /// specification expected by the application.]
        /// </summary>
        public const int BioAPIErrIncompatibleVersion = 0x00010a;


        /// <summary>
        /// The data in an input parameter is invalid.
        /// </summary>
        public const int BioAPIErrInvalidData = 0x00010b;


        /// <summary>
        /// The associated bsp is unable to capture raw samples from the requested BioAPI unit.
        /// </summary>
        public const int BioAPIErrUnableToCapture = 0x00010c;


        /// <summary>
        /// The associated bsp has no more space to allocated bir handles.
        /// </summary>
        public const int BioAPIErrTooManyHandles = 0x00010d;


        /// <summary>
        /// The function has been terminated because the timeout value has expired.
        /// </summary>
        public const int BioAPIErrTimeoutExpired = 0x00010e;


        /// <summary>
        /// The input bir is invalid for the purpose required.
        /// </summary>
        public const int BioAPIErrInvalidBir = 0x00010f;


        /// <summary>
        /// The associated bsp could not validate the signature on the bir.
        /// </summary>
        public const int BioAPIErrBirSignatureFailure = 0x000110;


        /// <summary>
        /// The associated bsp is unable to store the payload.
        /// </summary>
        public const int BioAPIErrUnableToStorePayload = 0x000111;


        /// <summary>
        /// The identity population is null.
        /// </summary>
        public const int BioAPIErrNoInputBirs = 0x000112;


        /// <summary>
        /// The associated bsp does not support the bdb requests.
        /// </summary>
        public const int BioAPIErrUnsupportedFormat = 0x000113;


        /// <summary>
        /// The associated bsp was unable to construct a bir from the input data.
        /// </summary>
        public const int BioAPIErrUnableToImport = 0x000114;


        /// <summary>
        /// The purpose recorded in the bir and/or the requested purpose are inconsistent with the function being performed.
        /// </summary>
        public const int BioAPIErrInconsistentPurpose = 0x000115;


        /// <summary>
        /// The function requires a fully processed bir.
        /// </summary>
        public const int BioAPIErrBirNotFullyProcessed = 0x000116;


        /// <summary>
        /// The bsp does not support the requested purpose.
        /// </summary>
        public const int BioAPIErrPurposeNotSupported = 0x000117;


        /// <summary>
        /// User cancelled operation before completion or timeout.
        /// </summary>
        public const int BioAPIErrUserCancelled = 0x000118;


        /// <summary>
        /// Bsp (or BioAPI unit attached to bsp) is currently being used by another biometric application.
        /// </summary>
        public const int BioAPIErrUnitInUse = 0x000119;


        /// <summary>
        /// The given bsp handle is invalid
        /// </summary>
        public const int BioAPIErrInvalidBspHandle = 0x00011a;


        /// <summary>
        /// A function has been called without initialized the BioAPI framework.
        /// </summary>
        public const int BioAPIErrFrameworkNotInitialized = 0x00011b;


        /// <summary>
        /// Bir handle is invalid (does not exist or has not been released).
        /// </summary>
        public const int BioAPIErrInvalidBirHandle = 0x00011c;


        /// <summary>
        /// The attempted calibration of a sensor unit was not able to be successfully completed.
        /// </summary>
        public const int BioAPIErrCalibrationNotSuccessful = 0x00011d;


        /// <summary>
        /// No preset bir population has been established.
        /// </summary>
        public const int BioAPIErrPresetBirDoesNotExist = 0x00011e;


        /// <summary>
        /// The bsp could not decrypt an input bir (and thus was unable to use it for the related operation).
        /// </summary>
        public const int BioAPIErrDecryptionFailure = 0x00011f;


        /// <summary>
        /// A reference to the file or the component being loaded cannot be found.
        /// </summary>
        public const int BioAPIErrComponentFileRefNotFound = 0x000201;


        /// <summary>
        /// Framework was unable to successfully load the bsp.
        /// </summary>
        public const int BioAPIErrBspLoadFail = 0x000202;


        /// <summary>
        /// Bsp for which an action was requested is not loaded.
        /// </summary>
        public const int BioAPIErrBspNotLoaded = 0x000203;


        /// <summary>
        /// BioAPI unit for which an action was not requested is not in the inserted state.
        /// </summary>
        public const int BioAPIErrUnitNotInserted = 0x000204;


        /// <summary>
        /// An invalid BioAPI unit id was requested.
        /// </summary>
        public const int BioAPIErrInvalidUnitID = 0x000205;


        /// <summary>
        /// An invalid category of bfp or BioAPI unit was requested.
        /// </summary>
        public const int BioAPIErrInvalidCategory = 0x000206;


        /// <summary>
        /// Invalid database handle.
        /// </summary>
        public const int BioAPIErrInvalidDbHandle = 0x000300;


        /// <summary>
        /// The associated bsp is unable to open the specified database.
        /// </summary>
        public const int BioAPIErrUnableToOpenDatabase = 0x000301;


        /// <summary>
        /// The database cannot be opened for the access requested because it is locked.
        /// </summary>
        public const int BioAPIErrDatabaseIsLocked = 0x000302;


        /// <summary>
        /// The specified database does not exist.
        /// </summary>
        public const int BioAPIErrDatabaseDoesNotExist = 0x000303;


        /// <summary>
        /// Create failed because the database already exists.
        /// </summary>
        public const int BioAPIErrDatabaseAlreadyExists = 0x000304;


        /// <summary>
        /// Invalid database name (uuid)
        /// </summary>
        public const int BioAPIErrInvalidDatabaseName = 0x000305;


        /// <summary>
        /// No record exists with the requested key.
        /// </summary>
        public const int BioAPIErrRecordNotFound = 0x000306;


        /// <summary>
        /// The specified marker handled is invalid.
        /// </summary>
        public const int BioAPIErrMarkerHandleIsInvalid = 0x000307;


        /// <summary>
        /// The database is already open.
        /// </summary>
        public const int BioAPIErrDatabaseIsOpen = 0x000308;


        /// <summary>
        /// Unrecognized access type.
        /// </summary>
        public const int BioAPIErrInvalidAccessRequest = 0x000309;


        /// <summary>
        /// End of database has been reached.
        /// </summary>
        public const int BioAPIErrEndOfDatabase = 0x00030a;


        /// <summary>
        /// The associated bsp cannot create the database.
        /// </summary>
        public const int BioAPIErrUnableToCreateDatabase = 0x00030b;


        /// <summary>
        /// The associated bsp cannot close the database.
        /// </summary>
        public const int BioAPIErrUnableToCloseDatabase = 0x00030c;


        /// <summary>
        /// The associated bsp cannot delete the database.
        /// </summary>
        public const int BioAPIErrUnableToDeleteDatabase = 0x00030d;


        /// <summary>
        /// The specified database is corrupt.
        /// </summary>
        public const int BioAPIErrDatabaseIsCorrupt = 0x00030e;


        /// <summary>
        /// A general location error.
        /// </summary>
        public const int BioAPIErrLocationError = 0x000400;


        /// <summary>
        /// Invalid horizontal or vertical postion.
        /// </summary>
        public const int BioAPIErrOutOfFrame = 0x000401;


        /// <summary>
        /// Invalid crosswise position.
        /// </summary>
        public const int BioAPIErrInvalidCrosswisePosition = 0x000402;


        /// <summary>
        /// Invalid lengthwise position.
        /// </summary>
        public const int BioAPIErrInvalidLengthwisePosition = 0x000403;


        /// <summary>
        /// Invalid distance.
        /// </summary>
        public const int BioAPIErrInvalidDistance = 0x000404;


        /// <summary>
        /// The position was too far to the right.
        /// </summary>
        public const int BioAPIErrLocationTooRight = 0x000405;


        /// <summary>
        /// The position was too far to the left.
        /// </summary>
        public const int BioAPIErrLocationTooLeft = 0x000406;


        /// <summary>
        /// The position was too high.
        /// </summary>
        public const int BioAPIErrLocationTooHigh = 0x000407;


        /// <summary>
        /// The position was too low.
        /// </summary>
        public const int BioAPIErrLocationTooLow = 0x000408;


        /// <summary>
        /// The position was too far away.
        /// </summary>
        public const int BioAPIErrLocationTooFar = 0x000409;


        /// <summary>
        /// The position was too near (close).
        /// </summary>
        public const int BioAPIErrLocationTooNear = 0x00040a;


        /// <summary>
        /// The position was too far foward.
        /// </summary>
        public const int BioAPIErrLocationTooForward = 0x00040b;


        /// <summary>
        /// The position was too far backward
        /// </summary>
        public const int BioAPIErrLocationTooBackward = 0x00040c;


        /// <summary>
        /// Sample quality is too poor for the operation to succeed. note: quality errors can be returned from any function which received a
        /// BioAPI bir input.
        /// </summary>
        public const int BioAPIErrQualityError = 0x000501;

    }
}
