﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BioAPI.Data;

namespace BioAPI
{
    /// <summary>
    /// Represents the group of biometric operations that are available when the unit of COMPARISON
    /// category is linked to the attach session.
    /// </summary>
    /// <author>
    /// See CREDITS.txt
    /// </author>
    public interface IComparison
    {
        UnitSchema UnitSchema { get; }
        byte[] ACBioInstance { get; }
        int FMRAchieved { get; }
        BIR AdaptedBIR { get; }
        byte[] Payload { get; }

        /// <summary>
        /// Performs biometric identification of the existing biometric sample. This function performs an identification
        /// (1-to-many) compare between a processed BIR and a set of reference BIRs. The input is the processed BIR captured
        /// specifically for this identification. The population that the compare takes place against can be presented in one
        /// of two ways:
        /// In a BIR database. Such population is defined via the Archive.NewIdentifyPopulation(string).
        /// Input in a collection of BIRs. Such population is created via the DataFactory.NewIdentifyPopulation(Collection)
        /// When the identify population is not explicitly specified (the 'population' parameter is null) the identify
        /// population that is set by the previous Comparison.PresetIdentifyPopulation() will be used.
        /// </summary>

        List<Candidate> Identify(int maxFMRrequested, BIR processedBIR, bool binning, int maxResults, int timeout);

        List<Candidate> Identify(int maxFMRrequested, BIR processedBIR, List<BIR> auxiliaryBIRs, bool binning, int maxResults, int timeOut);

        /// <summary>
        /// Provides the population of BIRs to the comparison unit. After this method is invoked successfully, an application
        /// can call AttachSession.Identify() or Comparison.Identify() specifying null for the 'population' parameter.
        /// The BSP keeps this setting in effect until the PresetIdentifyPopulation() is invoked with different setting or
        /// AttachSession.Terminate() is called. The enrolment population to be compared against can be presented in one of two ways:
        /// In a BIR database. Such population is defined via the Archive.NewIdentifyPopulation(string).
        /// Input in a collection of BIRs. Such population is created via the DataFactory.NewIdentifyPopulation(Collection)
        /// </summary>
        /// <param name="population">
        /// the population of BIRs against which the identification is performed.
        /// </param>
        void PresetIdentifyPopulation(IdentifyPopulation population);

        /// <summary>
        /// Performs biometric verification of an existing biometric sample. This function performs a verification (1-to-1)
        /// compare between two BIRs: the input BIR and the reference template. The input BIR is the processed BIR constructed
        /// specifically for this verification. The reference template was created at enrolment. The application shall
        /// request a maximum FMR value criterion (threshold) for a successful compare. The boolean VerifyResult.GetResult()
        /// indicates whether verification was successful or not, and the VerifyResult.GetFMRAchieved() is a FMR value
        /// (score) indicating how closely the BIRs actually compared.
        /// By setting the RequestAdaptedBir option the application can request that a BIR be constructed by adapting the
        /// reference template using the input processed BIR. If the compare is successful, an attempt may be made to adapt the
        /// reference template with information taken from the input BIR. (Not all BSPs perform adaptation). The resulting
        /// adapted BIR should now be considered an optimal enrolment template (it is up to the application whether it uses
        /// or discards this data.)
        /// It is important to note that adaptation may not occur in all cases.
        /// If a payload is associated with the reference template, the payload may be returned upon successful verification
        /// if the achieved FMR is sufficiently stringent; this is controlled by the policy of the BSP and specified in its
        /// schema. The return of payload is requested by setting the RequestPayload option.
        /// </summary>
        /// <param name="maxFMRrequested">
        /// the requested FMR criterion for successful verification (i.e., the comparison threshold).
        /// </param>
        /// <param name="processedBIR">
        /// the BIR to be verified.
        /// </param>
        /// <param name="referenceTemplate">
        /// the BIR to be verified against.
        /// </param>
        /// <param name="options">
        /// requests additional output such as adapted BIR and/or the payload.
        /// </param>
        /// <returns>
        /// the VerifyResult object that represents the result of the biometric operation.
        /// </returns>
        bool Verify(int maxFMRrequested, BIR processedBIR, BIR referenceTemplate, List<ResultOptions> options);

        bool Verify(int maxFMRrequested, BIR processedBIR, BIR referenceTemplate, List<BIR> auxiliaryBIRs, List<ResultOptions> options);
    }
}
