﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BioAPI.Data;

namespace BioAPI
{
    /// <summary>
    /// Represents the group of biometric operations that are available when the unit of PROCESSING category
    /// is linked to the attach session.
    /// </summary>
    /// <author>
    /// See CREDITS.txt
    /// </author>
    public interface IProcessing
    {
        UnitSchema UnitSchema { get; }
        byte[] ACBioInstance { get; }

        /// <summary>
        /// Takes a BIR containing biometric data in intermediate form for the purpose of creating a new enrolment template.
        /// A new BIR is constructed from the captured BIR, and (optionally) it may perform an update based on an existing
        /// reference template. The old reference template remains unchanged. The optional input reference template is
        /// provided for use in creating the new template, if the BSP supports this capability. When present, use of the
        /// input reference template by the BSP, to create the output template is optional. If the BSP supports an internal
        /// or BSP- controlled BIR database (e.g., smartcard or identification engine), it may optionally return the UUID
        /// assigned to the newly created template as stored within that BSP controlled BIR database. The UUID value shall
        /// be the same as that included in the BIR header, if present.
        /// </summary>
        /// <param name="capturedBIR">
        /// the captured BIR.
        /// </param>
        /// <param name="referenceTemplate">
        /// optionally, the existing template to be updated.
        /// </param>
        /// <param name="outputFormat">
        /// specifies which BDB format to use for the returned processed BIR, if the BSP supports more than one format.
        /// null indicates that the BSP is to select the format.
        /// </param>
        /// <param name="payload">
        /// a payload that will be stored by the BSP.
        /// </param>
        /// <returns>
        /// the CreateTemplateResult object that represents the result of the operation.
        /// </returns>
        BIR CreateTemplate(BIR capturedBIR, BIR referenceTemplate, DataFormat outputFormat, byte[] payload);

        BIR CreateTemplate(List<BIR> capturedBIRs, BIR referenceTemplate, DataFormat outputFormat, byte[] payload);
        
        /// <summary>
        /// Processes the intermediate data captured via a call to Sensor.Capture() for the purpose of either verification
        /// or identification. The BSP builds a "processed biometric sample" BIR.
        /// </summary>
        /// <param name="capturedBIR">
        /// the captured BIR.
        /// </param>
        /// <param name="outputFormat">
        /// specifies which BDB format to use for the returned processed BIR, if the BSP supports more than one format.
        /// null indicates that the BSP is to select the format.
        /// </param>
        /// <returns>
        /// the ProcessResult object that represents the result of processing.
        /// </returns>
        BIR Process(BIR capturedBIR, DataFormat outputFormat);

        /// <summary>
        /// Processes the intermediate data previously captured via a call to Sensor.Capture() in conjunction with auxiliary
        /// data, creating processed biometric samples for the purpose of subsequent verification or identification. It enables
        /// implementations that require the input of auxiliary data to the process operation. This capability may be used to
        /// support biometric compare-on-card or match-on-card (MOC). If the processing with auxiliary data capability is supported by the attached
        /// BSP invocation, the BSP builds a "processed biometric sample" BIR, otherwise an exception is thrown.
        /// NOTE 1: An example of auxiliary data is information related to the enrolment template which allows the processing
        /// operation to properly crop an input image to maximize the possibility of a subsequent compare (e.g., to ensure that
        /// the processed BIR for verification and the enrolment template are derived from the same portion of a finger).
        /// Another would be passing of biometric algorithm parameters.
        /// NOTE 2: The content and format of the auxiliary data are specified by the BIR Biometric Data Format field in the
        /// auxiliary BIR header and may be specific to a BSP.
        /// </summary>
        BIR Process(BIR capturedBIR, List<BIR> auxiliaryBIRs, DataFormat outputFormat);

   }
}
